/***************************************************************************
 *   Copyright (C) 2004 by Diego 'Flameeyes' Petteno                       *
 *   dgp85@users.sourceforge.net                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef KDIGESTDIGESTFILE_H
#define KDIGESTDIGESTFILE_H

#include <qobject.h>
#include <qptrlist.h>
#include <qstring.h>
#include <qfile.h>

#include <kmessagebox.h>

#include <stdint.h>

namespace KDigest {

/// Types of digests
enum DigestType {
	dtSFV,		///< SFV digest (CRC32 algorithm with comments support)
	dtMD5SUM	///< MD5SUM digest (MD5 algorithm, no comments support)
};

/// Flags for non-interactive digest check
enum NICflags {
	nicNone		= 0x00,	///< No flags
	nicConsole	= 0x01,	///< Use console instead of message boxes for output
	nicNoOutput	= 0x02,	///< Suppress all the output to console or messages
	nicFinalReport	= 0x04,	///< Prints a status report after the check if check was failed
	nicError	= 0x08,	///< Prints an error instead of an information
};

/**
@author Diego 'Flameeyes' Petten
@brief Base class for digest files handling

This class is the base class for all the class handling digest files, such
as SFV files or MD5SUMs.

@note should never be created directly!
*/
class DigestFile : public QObject
{
Q_OBJECT
public:
	~DigestFile();
	
	/// Digest entry status
	enum DigestStatus {
		dsNotChecked,		///< The file wasn't tested
		dsNotFound,		///< The file wasn't found
		dsErrorAccess,		///< Error Accessing the file
		dsBadDigest,		///< The file was found but its digest was different from the one declared
		dsOk			///< The file was found and its digest was exactly the one declared
	};
	
	/**
	@brief Base class for digest entries
	
	@note should never be created directly!
	*/
	class DigestEntry {
		public:
			virtual ~DigestEntry() { };
			QString filename;	///< Name of the file
			DigestStatus status;	///< Status of the entry
			virtual const QString digest() const = 0;
			virtual const QString actual() const = 0;
			virtual bool calculateActual(const QString &filename) = 0;
			virtual void writeOnFile(QFile &file) = 0;
		protected:
			DigestEntry() { };
	};

	/// A list of pointer to DigestEntry entries
	typedef QPtrList<DigestEntry> DigestEntries;

	/// Gets the current m_entries list
	inline const DigestEntries entries() const
	{ return m_entries; }
	
	virtual bool saveDigest(const QString &filename);
	
	virtual void addFile(const QString &filename) = 0;
	
	virtual bool testFile(const QString &path);
	static bool checkFile(DigestType dt, const QString &sfvfilename, uint8_t flags);
protected:
	DigestFile(QObject *parent = 0, const char *name = 0);
	
	DigestEntries m_entries;	///< Entries in the file
	static uint8_t m_buffer[4096];	///< Buffer to calculate the digest

	/**
	@brief Non-interactive check helper function to output a message
	@param message Message to output
	@param caption Caption of the (eventual) message box
	@param flags Flags for the check, @see checkFile() function, plus the flag for error or outputs
	*/
	inline static void nicOutput(const QString &message, const QString &caption, uint8_t flags)
	{
		if ( flags & nicNoOutput )
			return;
		
		if ( flags & nicError )
		{
			if ( flags & nicConsole )
				qWarning( "%s", message.latin1() );
			else
				KMessageBox::error(NULL, message, caption);
		} else {
			if ( flags & nicConsole )
				fprintf( stdout, message.latin1() );
			else
				KMessageBox::information(NULL, message, caption);
		}
	}
	
signals:
	/**
	@brief Signal emitted after a repainting cycle
	@param index Index in which the cycle is (the called signal slot should know the total cycles)
	*/
	void cycleCompleted(int index);
};

};

using KDigest::DigestFile;

#endif
